@@ -51,6 +51,16 @@ public interface RaftServerConfigKeys {
     setFiles(properties::setFiles, STORAGE_DIR_KEY, storageDir);
   }
 
+  String STORAGE_FREE_SPACE_MIN_KEY = PREFIX + ".storage.free-space.min";
+  SizeInBytes STORAGE_FREE_SPACE_MIN_DEFAULT = SizeInBytes.valueOf("0MB");
+  static SizeInBytes storageFreeSpaceMin(RaftProperties properties) {
+    return getSizeInBytes(properties::getSizeInBytes,
+        STORAGE_FREE_SPACE_MIN_KEY, STORAGE_FREE_SPACE_MIN_DEFAULT, getDefaultLog());
+  }
+  static void setStorageFreeSpaceMin(RaftProperties properties, SizeInBytes storageFreeSpaceMin) {
+    setSizeInBytes(properties::set, STORAGE_FREE_SPACE_MIN_KEY, storageFreeSpaceMin);
+  }
+
   String REMOVED_GROUPS_DIR_KEY = PREFIX + ".removed.groups.dir";
   File REMOVED_GROUPS_DIR_DEFAULT = new File("/tmp/raft-server/removed-groups/");
   static File removedGroupsDir(RaftProperties properties) {
@@ -421,7 +431,7 @@ public interface RaftServerConfigKeys {
     String PREFIX = RaftServerConfigKeys.PREFIX + ".data-stream";
 
     String ASYNC_REQUEST_THREAD_POOL_SIZE_KEY = PREFIX + ".async.request.thread.pool.size";
-    int ASYNC_REQUEST_THREAD_POOL_SIZE_DEFAULT = 16;
+    int ASYNC_REQUEST_THREAD_POOL_SIZE_DEFAULT = 32;
 
     static int asyncRequestThreadPoolSize(RaftProperties properties) {
       return getInt(properties::getInt, ASYNC_REQUEST_THREAD_POOL_SIZE_KEY,
@@ -445,6 +455,19 @@ public interface RaftServerConfigKeys {
     static void setAsyncWriteThreadPoolSize(RaftProperties properties, int port) {
       setInt(properties::setInt, ASYNC_WRITE_THREAD_POOL_SIZE_KEY, port);
     }
+
+    String CLIENT_POOL_SIZE_KEY = PREFIX + ".client.pool.size";
+    int CLIENT_POOL_SIZE_DEFAULT = 10;
+
+    static int clientPoolSize(RaftProperties properties) {
+      return getInt(properties::getInt, CLIENT_POOL_SIZE_KEY,
+          CLIENT_POOL_SIZE_DEFAULT, getDefaultLog(),
+          requireMin(0), requireMax(65536));
+    }
+
+    static void setClientPoolSize(RaftProperties properties, int num) {
+      setInt(properties::setInt, CLIENT_POOL_SIZE_KEY, num);
+    }
   }
 
   /** server rpc timeout related */
